//******************************************************************************
// Customizations of ../toolkit/customizeToolbar.js functions, as of changeset:
//
// 847a825087f2 2011-02-04 09:09 +0100
//
//******************************************************************************

var gNavToolbox;
var gToolboxes = [];
var gBrowserWindow;

onLoad = function() {
  let wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].
           getService(Components.interfaces.nsIWindowMediator);
  gBrowserWindow = wm.getMostRecentWindow("navigator:browser");

  gNavToolbox = gBrowserWindow.gNavToolbox;

  if ("arguments" in window && window.arguments[0]) {
    InitWithToolbox(window.arguments[0]);
    repositionDialog(window);
  }
  else if (window.frameElement &&
           "toolbox" in window.frameElement) {
    gToolboxSheet = true;
    InitWithToolbox(window.frameElement.toolbox);
    repositionDialog(window.frameElement.panel);
  }
  else {
    InitWithToolbox(gBrowserWindow.gNavToolbox);
  }
}

var fx_InitWithToolbox = InitWithToolbox;
InitWithToolbox = function(aToolbox) {
    fx_InitWithToolbox(aToolbox);

    // All toolboxes.
    let toolboxes = gToolboxDocument.getElementsByTagName("toolbox");
    for (let i = 0; i < toolboxes.length; ++i) {
      gToolboxes.push(toolboxes[i]);
      toolboxes[i].setAttribute("customizing", true);
    }
  }

repositionDialog = function(aWindow) {
  // Always use persisted dimensions and position!
  return;
}

/**
 * Persist the current set of buttons in all customizable toolbars to
 * localstore.
 */
function persistCurrentSets() {
  if (!gToolboxChanged || gToolboxDocument.defaultView.closed)
    return;

  // Need to loop through toolboxes to maintain customCount properly, rather
  // than use default function for toolbars and externalToolbars.
  gToolboxes.forEach(function(toolbox) {
    // Create array of toolbars in this toolbox.
    var customizableToolbarsForToolbox =
        Array.filter(toolbox.childNodes, isCustomizableToolbar);
    // Need to add addon-bar explicitly if toolbox is navigator-toolbox.
    if (toolbox.id == "navigator-toolbox")
      customizableToolbarsForToolbox.push(gToolboxDocument.getElementById("addon-bar"));

    var customCount = 0;
    customizableToolbarsForToolbox.forEach(function (toolbar) {
      // Calculate currentset and store it in the attribute.
      var currentSet = toolbar.currentSet;
      toolbar.setAttribute("currentset", currentSet);
//debugTT("toolbarid: "+toolbar.id);

      var customIndex = toolbar.hasAttribute("customindex");
//debugTT("toolboxid: "+toolbox.id);
      if (customIndex) {
//debugTT("customindex toolbarid: "+toolbar.id);
        if (!toolbar.hasChildNodes()) {
          // Remove any options from root doc.
          gBrowserWindow.TT.handleOptions("remove", toolbar, gToolbox);
          // Remove custom toolbars whose contents have been removed from
          // DOM and recreate externalToolbars list.
          let newExternalToolbars = [];
          gToolbox.externalToolbars.forEach(function (extToolbar, index) {
            if (extToolbar.id != toolbar.id)
              newExternalToolbars.push(extToolbar);
          });
          gToolbox.externalToolbars = newExternalToolbars;
          toolbox.removeChild(toolbar);
          --toolbox.customToolbarCount;
        }
        else {
          // Persist custom toolbar info on the <toolbarset/>
          toolbox.toolbarset.setAttribute("toolbar" + (++customCount),
              toolbar.toolbarName + ":" + currentSet);
          toolbar.setAttribute("customindex", customCount); //adjust count
          gToolboxDocument.persist(toolbox.toolbarset.id,
              "toolbar" + customCount);
//debugTT("persist customindex toolbar.id: "+toolbar.id);
        }
      }

      if (!customIndex) {
        // Persist the currentset attribute directly on hardcoded toolbars.
        gToolboxDocument.persist(toolbar.id, "currentset");
      }
    });

    // Remove toolbarX attributes from toolbarset for removed toolbars.
    while (toolbox.toolbarset &&
           toolbox.toolbarset.hasAttribute("toolbar"+(++customCount))) {
      toolbox.toolbarset.removeAttribute("toolbar"+customCount);
      gToolboxDocument.persist(toolbox.toolbarset.id, "toolbar"+customCount);
    }

    toolbox.removeAttribute("customizing");
  });
}

/**
 * Wraps an item that has been cloned from a template and adds
 * it to the end of a row in the palette.
 */
function wrapPaletteItem(aPaletteItem, aCurrentRow, aSpacer) {
  var wrapper = createWrapper(aPaletteItem.id, document);

//  wrapper.setAttribute("flex", 1);
  wrapper.setAttribute("align", "center");
  wrapper.setAttribute("pack", "center");
//  wrapper.setAttribute("minheight", "0");
//  wrapper.setAttribute("minwidth", "0");

  wrapper.appendChild(aPaletteItem);

  // XXX We need to call this AFTER the palette item has been appended
  // to the wrapper or else we crash dropping certain buttons on the
  // palette due to removal of the command and disabled attributes - JRH
  cleanUpItemForPalette(aPaletteItem, wrapper);

  aCurrentRow.appendChild(wrapper);
}

buildPalette = function() {
  var templateNode;

  var multiuseRow = document.getElementById("palette-row-multiuse");
  while (multiuseRow.lastChild)
    multiuseRow.removeChild(multiuseRow.lastChild);

  // Add the global multiuse (separator, spacer etc.) toolbar items first.
  templateNode = gNavToolbox.paletteMultiuse ? gNavToolbox.paletteMultiuse.firstChild :
                                               null;
  while (templateNode) {
    var paletteItem = templateNode.cloneNode(true);
    wrapPaletteItem(paletteItem, multiuseRow, null);
    templateNode = templateNode.nextSibling;
  };

  // Now, if the toolbox is not the global default, ie a custom toolbox from an
  // extension, see if it has any multiuse items.
  if (gToolbox.id != gNavToolbox.id) {
    templateNode = gToolbox.paletteMultiuse ? gToolbox.paletteMultiuse.firstChild :
                                              null;
    while (templateNode) {
      var paletteItem = templateNode.cloneNode(true);
      wrapPaletteItem(paletteItem, multiuseRow, null);
      templateNode = templateNode.nextSibling;
    };
  }

  var currentRow = document.getElementById("palette-row");
  while (currentRow.lastChild)
    currentRow.removeChild(currentRow.lastChild);

  // Add the single use palette items from the toolbox.
  var currentItems = getCurrentItemIds();
  templateNode = gToolbox.palette.firstChild;
  while (templateNode) {
    // Check if the item is already in a toolbar before adding it to the palette.
    // Need to check if a multiuse item due to bug in currentset setter where
    // Restore Default Set will append those items back to toolbar.palette,
    // thus making them appear in the list multiple times (Bug 354048 regression).
    if (!(templateNode.id in currentItems) &&
        !templateNode.hasAttribute("multiuseid")) {
      var paletteItem = document.importNode(templateNode, true);
      wrapPaletteItem(paletteItem, currentRow, null);
    }

    templateNode = templateNode.nextSibling;
  }

  var removedRow = document.getElementById("palette-row-removed");
  while (removedRow.lastChild)
    removedRow.removeChild(removedRow.lastChild);
}

/**
 * Creates a new palette item for a cloned template node and
 * adds it to the last slot in the palette.
 */
function appendPaletteItem(aItem) {
  var paletteRowRemoved = document.getElementById("palette-row-removed");
  wrapPaletteItem(aItem, paletteRowRemoved, null);
}

/* Improved to show drop indicator if dragging over a toolbar or hidden item */
function setDragActive(aItem, aValue) {
  var node = aItem;
  var direction = window.getComputedStyle(aItem, null).direction;
  var value = direction == "ltr"? "left" : "right";
  if (aItem.localName == "toolbar") {
    var child = aItem.lastChild;
    while (child && child.hidden)
      child = child.previousSibling;
    node = child || aItem;
    value = direction == "ltr"? "right" : "left";
  }

  if (!node)
    return;

  if (aValue) {
    if (!node.hasAttribute("dragover"))
      node.setAttribute("dragover", value);
  } else {
    node.removeAttribute("dragover");
  }
}

addNewToolbar = function () {
  var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                                .getService(Components.interfaces.nsIPromptService);

  var stringBundle = document.getElementById("stringBundle");
  var message = stringBundle.getString("enterToolbarName");
  var title = stringBundle.getString("enterToolbarTitle");

  var name = {};

  // Quitting from the toolbar dialog while the new toolbar prompt is up
  // can cause things to become unresponsive on the Mac. Until dialog modality
  // is fixed (395465), disable the "Done" button explicitly.
  var doneButton = document.getElementById("donebutton");
  if (doneButton)
    doneButton.disabled = true;

  while(true) {

    if (!promptService.prompt(window, title, message, name, null, {})) {
      if (doneButton)
        doneButton.disabled = false;
      return;
    }

    if (!name.value) {
      message = stringBundle.getFormattedString("enterToolbarBlank", [name.value]);
      continue;
    }
    //TODO: need better error msg string
    if (name.value.match(/[^\s|^\w]/g)) {
       message = stringBundle.getFormattedString("enterToolbarBlank", [name.value]);
       continue;
    }

    var dupeFound = false;

      // Check for an existing toolbar with the same display name
    forEachCustomizableToolbar(function (toolbar) {
      var toolbarName = toolbar.getAttribute("toolbarname");

      if (toolbarName == name.value &&
          toolbar.getAttribute("type") != "menubar" &&
          toolbar.nodeName == 'toolbar') {
        dupeFound = true;
        //break;
      }
    });

    if (!dupeFound)
      break;

    message = stringBundle.getFormattedString("enterToolbarDup", [name.value]);
  }

  var newToolbar = gToolbox.appendCustomToolbar(name.value, "", true);
  newToolbar.setAttribute("customizing", "true");

  toolboxChanged();

  if (doneButton)
    doneButton.disabled = false;
}

/**
 * Restore the default set of buttons to fixed toolbars,
 * remove all custom toolbars, and rebuild the palette.
 */
restoreDefaultSet = function () {
  // Unwrap the items on the toolbar.
  unwrapToolbarItems();

  // Remove all of the customized toolbars.
  forEachCustomizableToolbar(function (toolbar) {
    let customIndex = toolbar.getAttribute("customindex");
    if (customIndex) {
      // Clean up any customizations from the root doc.
      gBrowserWindow.TT.handleOptions("remove", toolbar, gToolbox);

      // Reset externalToolbars list.
      let newExternalToolbars = [];
      gToolbox.externalToolbars.forEach(function (extToolbar, index) {
        if (extToolbar.id != toolbar.id)
          newExternalToolbars.push(extToolbar);
      });
      gToolbox.externalToolbars = newExternalToolbars;

      let toolbox = toolbar.parentNode;
      toolbox.toolbarset.removeAttribute("toolbar"+customIndex);
      gToolboxDocument.persist(toolbox.toolbarset.id, "toolbar"+customIndex);
      toolbar.currentSet = "__empty";
      toolbox.removeChild(toolbar);
      --toolbox.customToolbarCount;
    }
  });

  // Restore the defaultset for fixed toolbars.
  forEachCustomizableToolbar(function (toolbar) {
    var defaultSet = toolbar.getAttribute("defaultset");
    if (defaultSet)
      toolbar.currentSet = defaultSet;

    // Remove any contextmenu options.
    gBrowserWindow.TT.handleOptions("remove", toolbar, gToolbox);
  });

  // Restore the default icon size and mode.
  document.getElementById("smallicons").checked = (updateIconSize() == "small");
  document.getElementById("modelist").value = updateToolbarMode();

  // Now rebuild the palette.
  buildPalette();

  // Now re-wrap the items on the toolbar.
  wrapToolbarItems();

  toolboxChanged("reset");
}

function updateToolboxProperty(aProp, aValue, aToolkitDefault) {
  var toolboxDefault = gToolbox.getAttribute("default" + aProp) ||
                       aToolkitDefault;

  gToolbox.setAttribute(aProp, aValue || toolboxDefault);
  gToolboxDocument.persist(gToolbox.id, aProp);

  forEachCustomizableToolbar(function (toolbar) {
//TT._log("toolbar.id: "+toolbar.id+" : "+aProp+" : "+toolboxDefault);
    var toolbarDefault = toolbar.getAttribute("default" + aProp) ||
                         toolboxDefault;
    if (toolbar.getAttribute("lock" + aProp) == "true" &&
        toolbar.getAttribute(aProp) == toolbarDefault)
      return;

    // For custom toolbars, set attribute on doc per other custom toolbar
    // handling in TT; don't persist here since these toolbars may go away
    // and never remove rdf traces.
    if (toolbar.hasAttribute("customindex")) {
//TT._log("customindex toolbar.id: "+toolbar.id+" : "+ aProp);
      toolbar.setToolbarProperty(aProp, aValue || toolboxDefault);
    }
    else {
      // Remove property from doc if set on fixed toolbars via contextmenu as
      // the global selection overrides (mode and iconsize).
      var attr = "_toolbar." + toolbar.id + "." + aProp;
      if (gToolboxDocument.documentElement.hasAttribute(attr)) {
        gToolboxDocument.documentElement.removeAttribute(attr);
        gToolboxDocument.persist(gToolboxDocument.documentElement.id, attr);
//TT._log("!customindex doc.id: "+gToolboxDocument.documentElement.id+" : "+ attr);
      }

      toolbar.setAttribute(aProp, aValue || toolbarDefault);
      gToolboxDocument.persist(toolbar.id, aProp);
    }
  });

  toolboxChanged(aProp);

  return aValue || toolboxDefault;
}

/* Until <statusbar> goes away */
function isToolbarItem(aElt) {
  return aElt.localName == "toolbarbutton" ||
         aElt.localName == "toolbaritem" ||
         aElt.localName == "toolbarseparator" ||
         aElt.localName == "toolbarspring" ||
         aElt.localName == "statusbar" ||
         aElt.localName == "toolbarspacer";
}

///////////////////////////////////////////////////////////////////////////
//// Drag and Drop observers

function onToolbarDrop(aEvent) {
  if (!gCurrentDragOverItem)
    return;

  setDragActive(gCurrentDragOverItem, false);

  var documentId = gToolboxDocument.documentElement.id;
  var draggedItemId = aEvent.dataTransfer.getData("text/toolbarwrapper-id/" + documentId);
  if (gCurrentDragOverItem.id == draggedItemId)
    return;

  var toolbar = aEvent.target;
  while (toolbar.localName != "toolbar")
    toolbar = toolbar.parentNode;

  var draggedPaletteWrapper = document.getElementById("wrapper-"+draggedItemId);
  if (!draggedPaletteWrapper) {
    // The wrapper has been dragged from the toolbar.
    // Get the wrapper from the toolbar document and make sure that
    // it isn't being dropped on itself.
    var wrapper = gToolboxDocument.getElementById("wrapper-"+draggedItemId);
    if (wrapper == gCurrentDragOverItem)
       return;

    // Don't allow non-removable kids (e.g., the menubar) to move.
    if (wrapper.firstChild.getAttribute("removable") != "true")
      return;

    // Remove the item from its place in the toolbar.
    wrapper.parentNode.removeChild(wrapper);

    // Determine which toolbar we are dropping on.
    var dropToolbar = null;
    if (gCurrentDragOverItem.localName == "toolbar")
      dropToolbar = gCurrentDragOverItem;
    else
      dropToolbar = gCurrentDragOverItem.parentNode;

    // Insert the item into the toolbar.
    if (gCurrentDragOverItem != dropToolbar)
      dropToolbar.insertBefore(wrapper, gCurrentDragOverItem);
    else
      dropToolbar.appendChild(wrapper);
  } else {
    // The item has been dragged from the palette

    // Create a new wrapper for the item. We don't know the id yet.
    var wrapper = createWrapper("", gToolboxDocument);
    var dropTarget = gCurrentDragOverItem == toolbar ? null : gCurrentDragOverItem;
    var multiUseItem = draggedPaletteWrapper.firstChild.hasAttribute("multiuseid");

    // Ask the toolbar to clone the item's template, place it inside the
    // wrapper, and insert it in the toolbar.
    var newItem = toolbar.insertItem(draggedItemId, dropTarget, wrapper);

    // Prepare the item and wrapper to look good on the toolbar.
    cleanupItemForToolbar(newItem, wrapper);
    wrapper.id = "wrapper-"+newItem.id;
    wrapper.flex = newItem.flex;

    // Remove the wrapper from the palette.
    var currentRow = draggedPaletteWrapper.parentNode;
    if (!multiUseItem)
      currentRow.removeChild(draggedPaletteWrapper);
  }

  gCurrentDragOverItem = null;

  toolboxChanged();
};

function onPaletteDrop(aEvent) {
  var documentId = gToolboxDocument.documentElement.id;
  var itemId = aEvent.dataTransfer.getData("text/toolbarwrapper-id/" + documentId);

  var wrapper = gToolboxDocument.getElementById("wrapper-"+itemId);
  if (wrapper) {
    // Don't allow non-removable kids (e.g., the menubar) to move.
    if (wrapper.firstChild.getAttribute("removable") != "true")
      return;

    var multiUseItem = wrapper.firstChild.hasAttribute("multiuseid");
    var wrapperType = wrapper.getAttribute("type");

    restoreItemForToolbar(wrapper.firstChild, wrapper);
    if (!multiUseItem) {
      appendPaletteItem(document.importNode(wrapper.firstChild, true));
      gToolbox.palette.appendChild(wrapper.firstChild);
    }

    // The item was dragged out of the toolbar.
    wrapper.parentNode.removeChild(wrapper);
  }

  toolboxChanged();
}
